local AceLocale = LibStub ("AceLocale-3.0")
local Loc = AceLocale:GetLocale ("Details_TimeLine")
local Graphics = LibStub:GetLibrary ("LibGraph-2.0")

--> create the plugin object
local TimeLine = _detalhes:NewPluginObject ("Details_TimeLine", DETAILSPLUGIN_ALWAYSENABLED)
tinsert (UISpecialFrames, "Details_TimeLine")
--> main frame (shortcut)
local TimeLineFrame = TimeLine.Frame

local debugmode = true

local type_cooldown = "cooldowns_timeline"

local function CreatePluginFrames()

	--> catch Details! main object
	local _detalhes = _G._detalhes
	local DetailsFrameWork = _detalhes.gump
	
	local all_types = {type_cooldown}
	local all_types_names = {Loc ["STRING_TYPE_COOLDOWN"]}
	local current_type = all_types [1]
	local current_segment = 1
	
	local GameCooltip = GameCooltip
	
	--> record if button is shown
	TimeLine.showing = false
	TimeLine.open = false
	
	--> record if boss window is open or not
	TimeLine.window_open = false
	
	--> shortcut for current combat
	local _combat_object
	local _is_in_combat = false
	
	--> timeline tables
	TimeLine.cooldowns_timeline = {}
	TimeLine.combat_data = {}
	TimeLine.rows = {}
	
	--> new combat
	function TimeLine:NewCombat()
		table.insert (TimeLine.cooldowns_timeline, 1, {})
		table.insert (TimeLine.combat_data, 1, {})
	end
	
	function TimeLine:FinishCombat()
	
		TimeLine.combat_data [1].total_time = _combat_object:GetCombatTime()
		local boss = _combat_object.is_boss
		if (boss) then
			TimeLine.combat_data [1].name = boss.name
		else
			TimeLine.combat_data [1].name = _combat_object.enemy
		end
		local _start, _end = _combat_object:GetDate()
		TimeLine.combat_data [1].date_start = _start
		TimeLine.combat_data [1].date_end = _end
		
		if (TimeLine.open) then
			TimeLine:Refresh()
		end
	end
	
	--> add data to a player
	function TimeLine:Add (capture_type, player, data)

		if (not _is_in_combat) then
			return
		end
		
		table.insert (data, 1, _combat_object:GetCombatTime())

		local player_table = TimeLine [capture_type] [1] [player]
		
		if (not player_table) then
			TimeLine [capture_type] [1] [player] = {}
			player_table = TimeLine [capture_type] [1] [player]
		end
		
		player_table [#player_table+1] = data
		
		local spellname = GetSpellInfo (data[3])
		print ("cooldown used:", player, spellname)
	end
	
	
	function TimeLine:OnDetailsEvent (event, ...)
		if (event == "HIDE") then --> plugin hidded, disabled
			self.open = false
		
		elseif (event == "SHOW") then --> plugin hidded, disabled
			self.open = true
			
		elseif (event == "COMBAT_PLAYER_ENTER") then --> combat started
		
			if (TimeLine.open) then
				TimeLine:HideIcon()
				TimeLine:CloseWindow()
			end
			
			_combat_object = select (1, ...)
			
			TimeLine:NewCombat()
			
			_is_in_combat = true
		
		elseif (event == "COMBAT_PLAYER_LEAVE") then
			TimeLine:FinishCombat()
			TimeLine:ShowIcon()
			_is_in_combat = false

		elseif (event == "DETAILS_DATA_RESET") then
			TimeLine:HideIcon()
			TimeLine:CloseWindow()
			
		end
	end
	
	--> show icon on toolbar
	function TimeLine:ShowIcon()
		TimeLine.showing = true
		--> [1] button to show [2] button animation: "star", "blink" or true (blink)
		TimeLine:ShowToolbarIcon (TimeLine.ToolbarButton, "star")
	end
	
	-->  hide icon on toolbar
	function TimeLine:HideIcon()
		TimeLine.showing = false
		TimeLine:HideToolbarIcon (TimeLine.ToolbarButton)
	end
	
	function TimeLine:DelaySegmentRefresh()
		for index, combat in ipairs (TimeLine.combat_data) do
			if (combat.name) then
				return Details_TimeLineSegmentDropdown.MyObject:Select (current_segment, true)
			end
		end
	end
	
	--> user clicked on button, need open or close window
	function TimeLine:OpenWindow()
		if (TimeLine.open) then
			return TimeLine:CloseWindow()
		else
			TimeLine.open = true
		end
		--> build all window data
		
		TimeLine:Refresh()

		TimeLine:ScheduleTimer ("DelaySegmentRefresh", 2)
		
		--> show
		TimeLineFrame:Show()
		return true
	end
	
	function TimeLine:CloseWindow()
		TimeLineFrame:Hide()
		TimeLine.open = false
		return true
	end

	--> create the button to show on toolbar [1] function OnClick [2] texture [3] tooltip [4] width or 14 [5] height or 14 [6] frame name or nil
	TimeLine.ToolbarButton = _detalhes.ToolBar:NewPluginToolbarButton (TimeLine.OpenWindow, [[Interface\CHATFRAME\ChatFrameExpandArrow]], Loc ["STRING_TOOLTIP"], 12, 12, "TIMELINE_BUTTON")
	--> setpoint anchors mod if needed
	TimeLine.ToolbarButton.y = 0
	TimeLine.ToolbarButton.x = 0

	--> build main frame
	TimeLineFrame:SetFrameStrata ("MEDIUM")
	TimeLineFrame:SetPoint ("center", UIParent, "center")
	
	TimeLineFrame:SetWidth (718)
	TimeLineFrame:SetHeight (414)
	TimeLineFrame:EnableMouse (true)
	TimeLineFrame:SetResizable (false)
	TimeLineFrame:SetMovable (true)
	TimeLineFrame:SetScript ("OnMouseDown", 
					function (self, botao)
						if (botao == "LeftButton") then
							self:StartMoving()
							self.isMoving = true
						end
					end)
					
	TimeLineFrame:SetScript ("OnMouseUp", 
					function (self)
						if (self.isMoving) then
							self:StopMovingOrSizing()
							self.isMoving = false
						end
					end)
	
	TimeLineFrame:SetBackdrop ({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 32,
		insets = {left = 5, right = 5, top = 5, bottom = 5}})
	TimeLineFrame:SetBackdropColor (.3, .3, .3, .3)

	local bottom_texture = DetailsFrameWork:NewImage (TimeLineFrame, nil, "$parentBottomTexture", nil, 702, 25, nil, "background")
	bottom_texture:SetTexture (0, 0, 0, .6)
	bottom_texture:SetPoint ("bottomleft", TimeLineFrame, "bottomleft", 8, 7)
	
	TimeLine.Times = {}
	for i = 1, 16 do 
		local time = DetailsFrameWork:NewLabel (TimeLineFrame, nil, "$parentTime"..i, nil, "00:00")
		time:SetPoint ("topleft", TimeLineFrame, "topleft", 78 + (i*36), -20)
		TimeLine.Times [i] = time
		local line = DetailsFrameWork:NewImage (TimeLineFrame, nil, "$parentTime"..i.."Bar", nil, 1, 361, nil, "border")
		line:SetTexture (1, 1, 1, .1)
		line:SetPoint ("topleft", time, "topleft")
	end
	
	function TimeLine:UpdateTimeLine (total_time)

		local time_div = total_time / 16 --786 -- 49.125
		
		for i = 1, 15 do
		
			local linha = TimeLine.Times [i+1]
			
			local this_time = time_div * i
			local minutos, segundos = math.floor (this_time / 60), math.floor (this_time % 60)
			
			if (segundos < 10) then
				segundos = "0" .. segundos
			end
			
			if (minutos > 0) then
				if (minutos < 10) then
					minutos = "0" .. minutos
				end
				linha:SetText (minutos .. ":" .. segundos)
			else
				linha:SetText ("00:" .. segundos)
			end
			
		end
		
	end
	
	--> dropdown select type (label)
		local select_type_label = DetailsFrameWork:NewLabel (TimeLineFrame, nil, "$parentTypeLabel", nil, Loc ["STRING_TYPE"])
		select_type_label:SetPoint ("bottomright", TimeLineFrame, "bottomright", -500, 15)
	--> dropdown select type (dropdown)
		local selectTypeOption = function (_, _, selected)
			current_type = selected
			TimeLine:Refresh()
		end
		
		local type_menu = {
			{value = type_cooldown, label = Loc ["STRING_TYPE_COOLDOWN"], onclick = selectTypeOption, icon = [[Interface\ICONS\Spell_Holy_GuardianSpirit]]}
		}
		local buildTypeMenu = function()
			return type_menu
		end
		
		local select_type_dropdown = DetailsFrameWork:NewDropDown (TimeLineFrame, nil, "$parentTypeDropdown", nil, 170, 20, buildTypeMenu, 1)
		select_type_dropdown:SetPoint ("left", select_type_label, "right", 2)
		
	--> dropdown select combat (label)
		local select_segment_label = DetailsFrameWork:NewLabel (TimeLineFrame, nil, "$parentSegmentLabel", nil, Loc ["STRING_SELECT_SEGMENT"])
		select_segment_label:SetPoint ("bottomright", TimeLineFrame, "bottomright", -200, 15)
	--> dropdown select combat (dropdown)
		local selectCombatOption = function (_, _, segment)
			current_segment = segment
			TimeLine:Refresh()
		end
		
		local buildCombatMenu = function()
			local t = {}
			for index, combat in ipairs (TimeLine.combat_data) do
				if (combat.name) then
					local minutos, segundos = math.floor (combat.total_time/60), math.floor (combat.total_time%60)
					t [#t+1] = {value = index, label = combat.name .. " (" .. index .. ")", onclick = selectCombatOption, icon = [[Interface\GROUPFRAME\UI-GROUP-MAINASSISTICON]], desc = minutos .. "m " .. segundos .. "s " .. " (" .. combat.date_start .. " - " .. combat.date_end .. ")"}
				end
			end
			return t
		end
		local select_segment_dropdown = DetailsFrameWork:NewDropDown (TimeLineFrame, nil, "$parentSegmentDropdown", nil, 170, 20, buildCombatMenu, nil)
		select_segment_dropdown:SetPoint ("left", select_segment_label, "right", 2)
	
	local backdrop_row = {bgFile = [[Interface\Tooltips\UI-Tooltip-Background]], tile = true, tileSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
	
	function TimeLine:HideRows()
		for i = 1, #TimeLine.rows do 
			local row = TimeLine.rows [i]
			row:Hide()
		end
	end	
	
	local top_bar = DetailsFrameWork:NewImage (TimeLineFrame, nil, "$parentRowTopLine", nil, 703, 1, nil, "overlay")
	top_bar:SetTexture (1, 1, 1, .4)
	local bottom_bar = DetailsFrameWork:NewImage (TimeLineFrame, nil, "$parentRowBottomLine", nil, 703, 1, nil, "overlay")
	bottom_bar:SetTexture (1, 1, 1, .4)

	local row_on_enter = function (self)
		top_bar:Show()
		bottom_bar:Show()
		top_bar:SetPoint ("bottom", self, "top")
		bottom_bar:SetPoint ("top", self, "bottom")
	end
	local row_on_leave = function (self)
		top_bar:Hide()
		bottom_bar:Hide()
	end
	
	local block_on_enter = function (self)
	
		self:SetBackdrop ({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
			edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", edgeSize = 4,
			insets = {left = 2, right = 2, top = 2, bottom = 2}})
		self:SetBackdropBorderColor (1, 1, 1, .9)
		self.texture:SetBlendMode ("ADD")
		
		top_bar:Show()
		bottom_bar:Show()
		
		local spell = self.spell
		local spell_name, _, spell_icon = GetSpellInfo (spell [1])
		
		GameCooltip:Reset()
		GameCooltip:AddLine (spell_name)
		GameCooltip:AddIcon (spell_icon, 1, 1, 14, 14)
		
		local minutos, segundos = math.floor (spell [2]/60), math.floor (spell [2]%60)
		GameCooltip:AddLine (Loc ["STRING_TIME"] .. ": " .. minutos .. "m " .. segundos .. "s ")

		GameCooltip:AddLine (Loc ["STRING_TARGET"] .. ": " .. spell [3])
		
		GameCooltip:ShowCooltip (self, "tooltip")
		
	end
	
	local block_on_leave = function (self)
	
		self:SetBackdrop (nil)
		self:SetBackdropBorderColor (0, 0, 0, 0)
		self.texture:SetBlendMode ("BLEND")
		
		top_bar:Hide()
		bottom_bar:Hide()
		
		GameCooltip:Hide()
		
	end

	function TimeLine:CreateSpellBlock (row)
	
		local block = CreateFrame ("frame", nil, row)
		block:SetFrameLevel (row.block_frame_level)
		row.block_frame_level = row.block_frame_level + 1
		if (row.block_frame_level > 9) then
			row.block_frame_level = 3
		end
		
		block.spell = {0, 0, ""} -- [1] spellid [2] used at [3] target name
		
		block:SetHeight (14)
		block:SetScript ("OnEnter", block_on_enter)
		block:SetScript ("OnLeave", block_on_leave)
		
		local texture = block:CreateTexture (nil, "background")
		texture:SetAllPoints (block)
		texture:SetTexture (0, 1, 0, .2)
		block.texture = texture
		
		return block
	end
	
	local SetSpellBlock = function (row, block, time_used, spellid, effect_time, target, total_time, pixel_per_sec)
	
		block.spell [1] = spellid
		block.spell [2] = time_used
		block.spell [3] = target
		
		-- largura onde comea a primeira linha: 114
		-- largura onde termina a ultima linha: 654
		-- largura total: 540
		
		local where = pixel_per_sec * time_used
		block:SetPoint ("left", row, "left", where + 106, 0)
		
		if (effect_time < 5) then
			effect_time = 5
		elseif (effect_time > 20) then
			effect_time = 20
		end
		block:SetWidth (effect_time * pixel_per_sec)
		
		--print ("where:",where, where+106, "tamanho:",effect_time, effect_time * pixel_per_sec)
		
	end
	
	--> player_table: [1] time [2] target [3] spellid
	local SetPlayer = function (row, player_name, player_table, total_time, pixel_per_sec)
	
		--> set name and class color
		row.name.text = player_name
		local _, class = UnitClass (player_name)
		if (class) then
			row.name.color = TimeLine.class_colors [class]
		else
			row.name.color = "white"
		end

		--> if showing cooldowns:
		if (current_type == type_cooldown) then
		
			--print (player_name,"usou",#player_table, "cooldowns")
			
			for spell_index, spell_table in ipairs (player_table) do 
				
				local time_used = spell_table [1]
				local target = spell_table [2]
				local spellid = spell_table [3]
				
				local spell_info = TimeLine.DefensiveCooldownSpells [ spellid ] or TimeLine.DefensiveCooldownSpellsNoBuff [ spellid ]
				local cooldown, effect_time = unpack (spell_info)
				
				local block = row.blocks [spell_index]
				if (not block) then
					row.blocks [spell_index] = TimeLine:CreateSpellBlock (row)
					block = row.blocks [spell_index]
				end
				
				SetSpellBlock (row, block, time_used, spellid, effect_time, target, total_time, pixel_per_sec)
			
			end
		end

	end
	
	function TimeLine:CreateRow()
		local index = #TimeLine.rows+1
		
		-- cria as labels e mouse overs e da o set point
		local f = CreateFrame ("frame", "DetailsTimeTimeRow"..index, TimeLineFrame)
		f:SetSize (703, 14)
		
		f:SetScript ("OnEnter", row_on_enter)
		f:SetScript ("OnLeave", row_on_leave)
		
		f.block_frame_level = 3
		
		f.name = DetailsFrameWork:NewLabel (f, nil, "$parentName", nil)
		f.name:SetPoint ("left", f, "left", 2, 0)
		
		f:SetBackdrop (backdrop_row)
		if (index%2 == 0) then
			f:SetBackdropColor (.1, .1, .1, .3)			
		else
			f:SetBackdropColor (.1, .1, .1, .4)
		end
		
		local height = (index * 14) + 22
		f:SetPoint ("topleft", TimeLineFrame, "topleft", 8, -height)
		
		f.blocks = {}

		TimeLine.rows [index] = f
		
		return f
	end
	
	function TimeLine:Refresh()
		-- quando abrir sempre ir pro ultimo combate
		
		if (_is_in_combat) then
			return
		end
		
		TimeLine:HideRows()
		
		local _table_to_use = TimeLine [current_type] [current_segment]
		local total_time = TimeLine.combat_data [current_segment].total_time
		local pixel_per_sec = 540 / total_time
		
		print ("total time:",total_time,"pixel per sec", pixel_per_sec)
		
		local i = 0
		
		for player_name, player_table in pairs (_table_to_use) do 
			
			i = i + 1
			
			local row = TimeLine.rows [i]
			if (not row) then
				row = TimeLine:CreateRow()
			end
		
			SetPlayer (row, player_name, player_table, total_time, pixel_per_sec)
			row:Show()
			
		end
		
		TimeLine:UpdateTimeLine (total_time)
		
	end
	
end

function TimeLine:OnCooldown (token, time, who_serial, who_name, who_flags, alvo_serial, alvo_name, alvo_flags, spellid, spellname)
	return TimeLine:Add (type_cooldown, who_name, {alvo_name, spellid})
end

function TimeLine:OnEvent (_, event, ...)

	if (event == "ADDON_LOADED") then
		local AddonName = select (1, ...)
		if (AddonName == "Details_TimeLine") then
			
			if (_G._detalhes) then
				
				--> create widgets
				CreatePluginFrames()

				--> core version required
				local MINIMAL_DETAILS_VERSION_REQUIRED = 10
				
				--> Install
				local install = _G._detalhes:InstallPlugin ("TOOLBAR", Loc ["STRING_PLUGIN_NAME"], "placeholder string", TimeLine, "DETAILS_PLUGIN_TIME_LINE", MINIMAL_DETAILS_VERSION_REQUIRED)
				if (type (install) == "table" and install.error) then
					print (install.error)
				end
				
				--> Register needed events
				_G._detalhes:RegisterEvent (TimeLine, "COMBAT_PLAYER_ENTER")
				_G._detalhes:RegisterEvent (TimeLine, "COMBAT_PLAYER_LEAVE")
				_G._detalhes:RegisterEvent (TimeLine, "DETAILS_DATA_RESET")
				
				_G._detalhes:InstallHook (DETAILS_HOOK_COOLDOWN, TimeLine.OnCooldown)
				
			end
		end
	end
end
